'use strict';

var BaseWebObservationReportView = require('app/base-web-observation-report-view');
var C = require('app/consts');
var SpinUtil = require('app/spin-util');
var App = require('app/app');
var MobilePhotoService = require('app/mobile-photo-service');

var AffirmativeReportPopup = Marionette.ItemView.extend({

    id: 'reportPopup',

    template: aur.tpl['affirmative-report-popup-view-tpl'],

    ui: {
        close: '.popupx',
        submit: '.affirmativeButton',
        cancel: '.negativeButton',
        startDate: '#start-date',
        startTime: '#start-time',
        endDate: '#end-date',
        endTime: '#end-time',
        searchInput: '#location',
        messages: '#web-observation-messages',
        selectPhoto: '#select-photo',
        removePhoto: '#remove-photo',
        takePhoto: '#take-photo',
        fileUpload: '#fileupload',
        obsImage: '#web-obs-image',
        obsImageContainer: '#web-obs-image-container',
        mobileCancel: '#mobile-cancel-web-report',
        mobileSubmit: '#mobile-save-web-report'
    },

    events: {
        'click @ui.mobileSubmit': 'submit',
        'click @ui.mobileCancel': 'closeView',
        'click @ui.mobileSave': 'submit',
        'click @ui.close': 'closeView',
        'click @ui.cancel': 'closeView',
        'click @ui.submit': 'submit',
        'click @ui.selectPhoto': 'selectPhoto',
        'click @ui.removePhoto': 'removePhoto',
        'click @ui.takePhoto': 'takeMobilePhoto',
        'blur input[type=date], input[type=checkbox], input[type=text], select, textarea': 'updateModel'
    },

    errorMsg: 'Oops, it looks like we had a problem saving your observation.',
    successMsg: "Thanks for the report!",

    onInitialize: function() {
        this.spin = new SpinUtil();
        this.model.set('see_aurora', true);
    },

    rendered: function() {
        if(!window.cordova) {
            this.ui.fileUpload.fileupload({
                url: C.get('host') + '/web-obs/image',
                add: _.bind(function (e, data) {
                    _.each(data.files, function(file) {
                        data.formData = {file_name: this.uniquifyFileName(file.name)};
                        this.model.set('file_name', data.formData.file_name);
                    }, this);

                    this.spin.spin(this.ui.obsImageContainer);
                    data.submit();
                }, this),
                done: _.bind(function(e, data) {
                    if(data.result.save_url){
                        this.handleTempImageUploaded(data.result.save_url);
                    }
                }, this),
                always: _.bind(function(e, data) {
                    this.spin.stop();
                }, this)
            });
        }
    },

    handleTempImageUploaded: function(saveUrl) {
        this.ui.obsImage.prop('src', saveUrl);
        this.model.set('imageUrl', saveUrl);
    },

    selectPhoto: function() {
        if(window.cordova) {
            this.selectMobilePhoto();
        } else {
            this.$('#fileupload').click();
        }
    },

    uniquifyFileName: function(filename) {
        return (new Date().getTime()) + '-' + filename;
    },

    mobileImageUploaded: function(imageData) {
        // Ignore the directories and just get us the filename, then uniquify
        var fileName = this.uniquifyFileName(imageData.split('/').reverse()[0]);
        var options = new FileUploadOptions();
        options.fileKey = "image";
        options.fileName = fileName;
        options.mimeType = "image/png";
        options.params = {file_name: fileName};

        this.model.set('file_name', fileName);
        this.model.set('imageUrl', imageData);
        this.ui.obsImage.attr('src', imageData);

        if(App.request('isConnected')) {
            MobilePhotoService.saveMobilePhoto(imageData, options);
        } else {
            this.imageToUpload = {imageData: imageData, imageOptions: options};
        }

    },

    mobileImageUploadFailed: function(message) {
        alert('Failed to get a picture: ' + message);
    },

    removePhoto: function() {
        this.model.set('file_name', null);
        this.model.set('imageUrl', null);
        this.ui.obsImage.attr('src', C.get('host') + '/static/img/img-placeholder.png');
    },

    selectMobilePhoto: function() {
        navigator.camera.getPicture(_.bind(this.mobileImageUploaded,this), _.bind(this.mobileImageUploadFailed, this), { 
            sourceType: Camera.PictureSourceType.SAVEDPHOTOALBUM,
            destinationType: Camera.DestinationType.FILE_URI,
            encodingType: Camera.EncodingType.PNG
        });
    },

    takeMobilePhoto: function() {
        navigator.camera.getPicture(_.bind(this.mobileImageUploaded,this), _.bind(this.mobileImageUploadFailed, this), { 
            destinationType: Camera.DestinationType.FILE_URI,
            encodingType: Camera.EncodingType.PNG
        });
    }
});

_.extend(AffirmativeReportPopup.prototype, BaseWebObservationReportView);

module.exports = AffirmativeReportPopup;