'use strict';

var App = require('app/app');
var Utils = require('app/utils');
var MapHelper = require('app/map-helper');

module.exports = {
	tagName: 'li',
    className: 'filter-toggle',

    onInitialize: function() {
        this.mapHelper = new MapHelper();
        this.currentMoment = moment();

        this.getLayerData();
 
        this.initHandlers();
        this.initExtraHandlers();
    },

	initExtraHandlers: function() {
    	this.listenTo(this.mapHelper, ('markerClicked'), function(data) {
            this._handleMarkerClicked(data);
        }.bind(this));

        App.vent.on('unhighlightSelected', function(newMarker) {
            this._handleUnhighlightedSelected(newMarker);
        }.bind(this));

        App.commands.setHandler(this.toggleMessage, function() {
            this.toggleLayer();
        }.bind(this));
        
        App.reqres.setHandler(this.isActiveMessage, function() {
            return this.isActive();
        }.bind(this));
    },

    isActive: function() {
        return this.el.classList.contains('active');
    },

    toggleLayer: function() {
        if(this.isActive()) {
            this.removeLayer();
            App.execute(this.updateMessage, true);
        } else {
            this.getLayerData();
            App.execute(this.updateMessage, false);
        }
    },

    _getTime: function() {
        var nowRounded = Utils.roundXMinutes(this.currentMoment.clone(), 1);
        return nowRounded.format(Utils.defaultDateFmtWithTZ());
    },

    _getLocalOffset: function() {
        return this.currentMoment.toDate().getTimezoneOffset() * 60;
    },

    _handleMarkerClicked: function(data) {
    	App.vent.trigger('unhighlightSelected', data.marker);
        var icon = {
            url: this.selectedIconUrl,
            scaledSize: new google.maps.Size(35, 35)
        };
        this.lastSelected = data.marker;
        this.mapHelper.updateIcon(data.marker, icon);

        App.vent.trigger(this.markerClickedEvent, data.content);
    },

    _handleUnhighlightedSelected: function(newMarker) {
    	if(this.lastSelected && newMarker !== this.lastSelected) {
            var icon = {
                url: this.unselectedIconUrl,
                scaledSize: new google.maps.Size(35, 35)
            };
            this.mapHelper.updateIcon(this.lastSelected, icon);
        }
    }

};