'use strict';

var BaseMapControlView = require('app/base-map-control-view');
var App = require('app/app');
var C = require('app/consts');

var BaseSightingFilterView = {

    initHandlers: function() {
        this.collection.bind('reset', this.showLayer.bind(this));

        this.listenTo(this.collection, "add", function() {
            this.getLayerData(1);
        }.bind(this));

        App.vent.on('mapDateChanged', function(dateString) {
            this.currentMoment = moment(dateString);
            if(this.isActive()) {
                this.removeLayer();
                this.getLayerData(1);
            }
        }.bind(this));
    },

    removeLayer: function() {
        this.mapHelper.removeCluster();
        this.el.classList.remove('active');
    },

    getLayerData: function(page) {
        if(!page) page = 1;

        this.collection.queryParams.end_date = this._getTime();
        this.collection.queryParams.local_offset = this._getLocalOffset();
        if(this.negSighting) this.collection.queryParams.neg_sighting = true;
        if(this.posSighting) this.collection.queryParams.pos_sighting = true;
        this.collection.getPage(page, {reset: true})
        .done(function() {
            if(this.collection.hasNextPage()) this.getLayerData(page + 1);
        }.bind(this));
        this.el.classList.add('active');
    },

    showLayer: function() {
        var observations = this.collection.toJSON(),
            options = {
            gridSize: C.get('CLUSTER_GRID_SIZE'),
            maxZoom: 15,
            minimumClusterSize: C.get('CLUSTER_SIZE'),
            styles: [{
                url: this.groupLayerIcon,
                width: 35,
                height: 35
            }],
        },
        markerOpts = {
            icon: {
                url: this.unselectedIconUrl,
                scaledSize: new google.maps.Size(35, 35)
            }
        };
        this.mapHelper.renderCluster(observations, options, markerOpts);
    }
};

_.extend(BaseSightingFilterView, BaseMapControlView);

module.exports = BaseSightingFilterView;