'use strict';

var BaseView = require('app/base-view');
var InputUpdater = require('app/input-updater');
var App = require('app/app');
var Utils = require('app/utils');
var LocationService = require('app/location-service');
var MobilePhotoService = require('app/mobile-photo-service');

var BaseWebObservationReportView = {

    initialize: function() {
        var userCoords;
        try {
             userCoords = App.request('userCoords');
        } catch(e) {
            return;
        }

        if(userCoords) {
            this.updateLocation(userCoords.coords);
        }
        this.bindValidation();  
        if(this.onInitialize) this.onInitialize();
    },

    onRender: function() {
        this.renderDateTimePickers();
        this.renderLocationInput();
        if(this.rendered) this.rendered();
    },

    onShow: function() {
        $('#reportPopupContainer').removeClass('fade-out-fast');
        $('#reportPopupContainer').addClass('fade-in-fast');
    },

    closeView: function() {
        if(window.cordova) {
            App.execute('routeLanding');
        } else {
            $('#reportPopupContainer').addClass('fade-out-fast');
            $('#reportPopupContainer').removeClass('fade-in-fast');
            setTimeout(function() {
                App.reportPopover.reset();
            }.bind(this), 160);
        }
    },

    onHandleSuccess: function() {
        this.ui.submit.remove();
        this.ui.cancel.val('Close');
        
        this.ui.mobileSubmit.remove();
        this.ui.mobileCancel.text('Close');
    },

    updateDateOrTime: function(typeToUpdate, pickerObj) {
        var newTime = pickerObj.get('select'),
            id = pickerObj.get('id');
        var fieldToUpdate = id.match(/start*/gi) ? 'time_start' : 'time_end';
        var oldTime = this.model.get(fieldToUpdate);
        var oldTimeMoment = moment(oldTime);

        if(typeToUpdate === 'date') {
            oldTimeMoment.year(newTime.year);
            oldTimeMoment.month(newTime.month);
            oldTimeMoment.date(newTime.date);
        } else {
            oldTimeMoment.hours(newTime.hour);
            oldTimeMoment.minutes(newTime.mins);
        }

        this.model.set(fieldToUpdate, oldTimeMoment.format(Utils.defaultDateFmtWithTZ()));
    },

    updateLocation: function(location) {
        if(window.cordova && !App.request('isConnected')) {
            window.localStorage.location = location;
        } else {
            $.when(LocationService.updateLocation(location))
            .then(_.bind(function(data) {
                this.model.set(data);
                this.render();
            }, this));
        }
    },

    renderDateTimePickers: function(){
        var self = this;
        var timeOptions = { 
            interval: 15,
            clear: false,
            onSet: function() { self.updateDateOrTime.call(self, 'time', this); }
        };
        var dateOptions = { 
            max: new Date(),
            format: 'd mmm yyyy',
            clear: false,
            onSet: function() { self.updateDateOrTime.call(self, 'date', this); }
        };

        this.ui.startDate.pickadate(dateOptions);
        this.ui.endDate.pickadate(dateOptions);
        this.ui.startTime.pickatime(timeOptions);
        this.ui.endTime.pickatime(timeOptions);
    },

    renderLocationInput: function() {
        if(window.cordova && !App.request('isConnected')) {
            this.ui.searchInput.on('change', _.bind(function() {
                this.location = this.ui.searchInput.val();
            }, this));
        } else {
            var input = this.ui.searchInput[0];
            var searchBox = new google.maps.places.SearchBox(input);

            google.maps.event.addListener(searchBox, 'places_changed', _.bind(function() {
                var place = searchBox.getPlaces()[0];

                if(window.cordova && !App.request('isConnected')) {
                    this.location = place.name;
                } else {
                    this.updateLocation(place);
                }

            }, this));
        }
    },

    updateModel: function () {
        this.updateFromCheckboxes();
        this.updateFromElementsWithVals();
    },

    onHandleError: function(model, response, options) {
        if(response.status === 429) return 'Sorry, you can only report 2/hour and 6/day';
    },

    submit: function(e) {
        e.preventDefault();

        if(window.cordova) {
            this._submitMobile();
        } else {
            this.instantSubmit();
        }
    },

    _submitMobile: function() {
        var isConnected = App.request('isConnected'),
            isImageToUpload = this.imageToUpload;

        if(isConnected && isImageToUpload) {
            $.when(MobilePhotoService.saveMobilePhoto())
            .then(_.bind(function(data) {
                this.handleTempImageUploaded(data.save_url);
                this.instantSubmit();
            }, this));
        } else if(isConnected) {
            this.instantSubmit();
        } else {
            var pending = window.localStorage.getItem('observationsPendingSave');
            if(!pending) pending = [];
            else pending = JSON.parse(pending);

            pending.push({
                json: this.model.toJSON(),
                imageToUpload: this.imageToUpload,
                location: this.location
            });

            window.localStorage.observationsPendingSave = JSON.stringify(pending);

            this.successMsg = 'Thanks for the report! It will be sumitted once you are connected to the internet';
            this.handleSuccess();
        }
    }
};

_.defaults(BaseWebObservationReportView, BaseView, InputUpdater);

module.exports = BaseWebObservationReportView;