'use strict';

var TwitterUtils = require('app/twitter-utils');
var App = require('app/app');
var C = require('app/consts');
var Dropdown = require('app/dropdown');
var HandlebarsHelpers = require('app/handlebars-helpers');

var Base = function() {
    window.Aur = window.Aur || {};
    window.Aur.authenticated = false;

    this.configAjax();
    this.initBackboneValidators()
    this.initPolyFills();
    this.initDropdowns();
    this.initHandlers();

    var helpers = new HandlebarsHelpers();
    helpers.registerHelpers();
};

_.extend(Base.prototype, {
    initDropups: function() {
        var $btns = $('button[data-toggle="dropup"]');
        $btns.on('click', function(e) {
            var $el = $(e.originalEvent.srcElement);
            if (_.contains($btns, $el)) {
                e.preventDefault();
                return;
            }
            var $parent = $el.closest('.btn-group');
            if ($parent.hasClass('open')) $parent.removeClass('open');
            else $parent.addClass('open');
        });
    },

    initDropdowns: function() {
        if(!window.cordova) {
            var navBar = new Dropdown();
            navBar.init($('.navBarIcon'), $('#navBar > ul'));

            var notificationItem = new Dropdown();
            notificationItem.init($('#notificationItem'), $('#notificationContainer'));
        }
    },

    configAjax: function() {
        $.ajaxSetup({
            headers: {
                'X-CSRFToken': $.cookie('csrftoken')
            }
        });
    },

    initBackboneValidators: function() {
        Backbone.Validation.configure({
            selector: 'id'
        });
        _.extend(Backbone.Validation.callbacks, {
            valid: function(view, attr, selector) {
                view.$('[data-field="'+ attr +'"]').removeClass('error');
            },
            invalid: function(view, attr, error, selector) {
                view.$('[data-field="'+ attr +'"]').addClass('error');
                view.$('#' + view.model.name + '-messages')
                    .addClass('error-text')
                    .removeClass('success-text')
                    .append('<p>' + error + '</p>');
                view.$('#' + view.model.name + '-messages')[0].scrollIntoView();
            }
        });
    },

    initPolyFills: function() {
        if(!Array.isArray) {
          Array.isArray = function(arg) {
            return Object.prototype.toString.call(arg) === '[object Array]';
          };
        }
    },

    initHandlers: function() {
        App.commands.setHandler('loginUser', _.bind(function(data) {
            Aur.authenticated = true;
            Aur.user = data.user;
            $.ajaxSetup({
                headers: {
                    'X-CSRFToken': data.csrf_token
                }
            });
        }, this));
    }
});

module.exports = Base;