'use strict';

var App = require('app/app');

var AGREE_BTN = 2;

module.exports = Marionette.ItemView.extend({
    className: 'row noMapContainer',
    template: aur.tpl['finish-registration-view-tpl'],

    privacyPolicyTemplate: aur.tpl['privacy-policy-tpl'],

    ui: {
        emailInput: '#email',
        locationInput: '#location',
        registrationSubmitBtn: '#registration-submit',
        privacyPolicyLabel: 'label[for="privacy_policy"]',
        notificationsCheckbox: '#notifications',
        plansTextArea: '#plans',
        aboutTextArea: '#about',
        nicknameInput: '#nickname',
        privacyPolicyCheckbox: '#privacy_policy'
    },

    events: {
    	'click @ui.registrationSubmitBtn': 'validateAndSave',
        'click @ui.privacyPolicyLabel': 'showPrivacyPolicy',
        'change @ui.emailInput': 'updateModel',
        'click @ui.privacyPolicyCheckbox': 'updateModel',
        'click @ui.notificationsCheckbox': 'updateModel',
        'change @ui.plansTextArea': 'updateModel',
        'change @ui.aboutTextArea': 'updateModel',
        'change @ui.nicknameInput': 'updateModel'
    },

    initialize: function() {
        Backbone.Validation.bind(this);
        if(!this.model.get('user')) this.model.set('user', {});
    },

    onRender: function() {
    	this.initLocationSearchInput();
    },

    initLocationSearchInput: function() {
        this.ui.locationInput.keypress(function(e) {
            if (e.which == 13) {
                return false;
            }
        });
    	var searchBox = new google.maps.places.SearchBox(this.ui.locationInput[0]);
        google.maps.event.addListener(searchBox, 'places_changed', function() {
            var place = searchBox.getPlaces()[0];
            var location = {
                address: place.formatted_address,
                name: place.name,
                latitude: place.geometry.location.lat(),
                longitude: place.geometry.location.lng(),
                google_reference: place.reference,
                google_id: place.id
            };

            // Set in two different locations. One for mobile, one for desktop
            this.model.set('location', location);
            this.model.set(location);
        }.bind(this));
    },

    updateModel: function() {
        var user = this.model.get('user');
        user.email = this.ui.emailInput.val();

        this.model.set('user', user);
        this.model.set('email', user.email);
        this.model.set('privacy_policy', this.ui.privacyPolicyCheckbox.is(':checked'));
        this.model.set('notifications', this.ui.notificationsCheckbox.is(':checked'));
        this.model.set('about', this.ui.aboutTextArea.val());
        this.model.set('plans', this.ui.plansTextArea.val());
        this.model.set('nickname', this.ui.nicknameInput.val());
    },

    showPrivacyPolicy: function(e) {
        if(window.cordova) {
            e.preventDefault();

            navigator.notification.confirm(this.privacyPolicyTemplate(), 
                _.bind(function(buttonIndex) {
                    if(buttonIndex === AGREE_BTN) {
                        this.model.set('privacy_policy', false);
                        this.ui.privacyPolicyCheckbox.prop('checked', null);
                    } else {
                        this.model.set('privacy_policy', true);
                        this.ui.privacyPolicyCheckbox.prop('checked', 'checked');
                    }
                }, this),
                'Privacy Policy',
                ['Agree', "Don't Agree"]
            );
        }
    },

    updateForm: function() {
        _.each(_.keys(this.model.toJSON()), function(key) {
            this.$('[name="' + key + '"]').val(this.model.get(key));
        }, this);
    },

    validateAndSave: function(e) {
        this.$('#' + this.model.name + '-messages').html('').removeClass('alert alert-danger');
        this.model.validate();

        if(window.cordova) {
            e.preventDefault();
            if(this.model.isValid()) {
                this.mobileSave();
            }
        } else if (this.model.isValid()) {
            this.updateForm();
            
        } else {
            e.preventDefault();
        }
    },

    mobileSave: function() {
        this.model.save().done(function(user) {
            Aur.authenticated = true;
            Aur.user = user;
            App.execute('routeSettings');
        });
    }
  
});