'user strict';

var C = require('app/consts');
var TwitterUtils = require('app/twitter-utils');

var HandlebarsHelpers = function() {};

HandlebarsHelpers.prototype.registerHelpers = function() {
    Handlebars.registerHelper('fmtDefault', function(date) {
        return moment(date).format('M/D/YY [at] h:mm a');
    });
    Handlebars.registerHelper('fmtDate', function(date) {
        return moment(date).format('D MMM YYYY');
    });
    Handlebars.registerHelper('fmtDateDayMonth', function(date) {
        return moment(date).format('MMM, Do');
    });
    Handlebars.registerHelper('fmtTime', function(date) {
        return moment(date).format('h:mm A');
    });
    Handlebars.registerHelper('isChecked', function(value, expected) {
        return this._printIfPresent(value, expected, 'checked');
    }.bind(this));

    Handlebars.registerHelper('isSelected', function(value, expected) {
       return this._printIfPresent(value, expected, 'selected'); 
    }.bind(this));

    Handlebars.registerHelper('fmtTwitterName', function(user_screen_name) {
        return '<a target="_blank" href="https://twitter.com/' + user_screen_name + '">@' + user_screen_name + '</a>'
    });

    Handlebars.registerHelper('fmtTwitterText', function(text) {
        return TwitterUtils.formatText(text);
    });

    Handlebars.registerHelper('fmtVerifiedTwtrObs', function(observation) {
        switch(observation.verified_type){
            case 'pos': return 'Tweet confirmed to be a positive sighting';
            case 'neg': return 'Tweet confirmed to be a negative sighting';
        }
    });

    Handlebars.registerHelper('fmtSeeAurora', function(seeAurora) {
        if(seeAurora) return 'User saw the aurora';
        return 'User didn\'t see the aurora';
    });

    Handlebars.registerHelper('fmtObsDetail', function(arr, str) {
        var result = '';
            reduced;

        if(_.isArray(arr)) {
            // Reduce the array into one comma seperated string
            var reduced = _.reduce(arr, function(memo, val) {
                if(memo === 0) return C.get(val);
                return memo + ', ' + C.get(val);
            }, 0);
        } else if(arr) {
            result = C.get(arr); // In this case, arr isn't an array, it's just a string
        }

        if(reduced) result = reduced + (str ? ', ' + str : '');
        return result;
    });

    Handlebars.registerHelper('ifCond', function(v1, v2, options) {
        if(_.isArray(v1)) v1 = v1.length;
        if(_.isArray(v2)) v2 = v2.length;
        if(v1 || v2) {
        return options.fn(this);
      }
      return options.inverse(this);
    });    

    Handlebars.registerHelper('ifIsMobile', function(options) {
        if(window.cordova){
            return options.fn(this);
        }
        return options.inverse(this);
    });

    Handlebars.registerHelper('getConst', function(c) {
        return C.get(c);
    });

    Handlebars.registerHelper('CSRF', function() {
        return $.cookie('csrftoken');
    });

    Handlebars.registerHelper('isUserOfSocialType', function(user, type, options) {
        if(user && user.social_auth && user.social_auth[0]) {
            if(user.social_auth[0].provider === type) {
                return options.fn(this);
            }
        }
        return false;
    });
};

HandlebarsHelpers.prototype._printIfPresent = function(value, expected, toPrint) {
    if(_.isArray(expected)) {
        return _.contains(expected, value) ? toPrint: '';
    }
    return value === expected ? toPrint : '';
}

module.exports = HandlebarsHelpers;