'use strict';

var App = require('app/app');
var Base = require('app/base');
var ScientistCollection = require('app/scientist-collection');
var WebObservationCollection = require('app/web-observation-collection');
var TwitterObservationCollection = require('app/twitter-observation-collection');
var Map = require('app/map');
var OvalForecastView = require('app/oval-forecast-view');
var OvalEstimateView = require('app/oval-estimate-view');
var DateAndTimePickerView = require('app/date-and-time-picker-view');
var AffirmativeReportPopupView = require('app/affirmative-report-popup-view');
var WebObservation = require('app/web-observation');
var NegativeReportPopupView = require('app/negative-report-popup-view');
var SightingHeatmapLayerView = require('app/sighting-heatmap-layer-view');
var OtherMapControlsView = require('app/other-map-controls-view');
var MapDrawerLayout = require('app/map-drawer-layout');
var SightingControlsView = require('app/sighting-controls-view');
var CurrentConditionsView = require('app/current-conditions-view');
var ScientistFilterView = require('app/scientist-filter-view');
var PositiveSightingFilterView = require('app/positive-sighting-filter-view');
var NegativeSightingFilterView = require('app/negative-sighting-filter-view');
var TwitterUnverifiedFilterView = require('app/twitter-unverified-filter-view');
var TwitterVerifiedFilterView = require('app/twitter-verified-filter-view');
var OvalForecastLegendView = require('app/oval-forecast-legend-view');
var OvalEstimateLegendView = require('app/oval-estimate-legend-view');
var CloudCoverView = require('app/cloud-cover-view');
var DayNightLayerView = require('app/day-night-layer-view');
var HelpPopupView = require('app/help-popup-view');
var MobileOvalForecastView = require('app/mobile-oval-forecast-view');
var MobileOvalEstimateView = require('app/mobile-oval-estimate-view');
var MobileCitizenEstimateView = require('app/mobile-citizen-estimate-view');
var MobileCloudCoverView = require('app/mobile-cloud-cover-view');
var MobileCurrentConditionsView = require('app/mobile-current-conditions-view');
var MobileDayNightLayerView = require('app/mobile-day-night-layer-view');
var MobilePositiveSightingFilterView = require('app/mobile-positive-sighting-filter-view');
var MobileNegativeSightingFilterView = require('app/mobile-negative-sighting-filter-view');
var MobileTwitterUnverifiedFilterView = require('app/mobile-twitter-unverified-filter-view');
var MobileTwitterVerifiedFilterView = require('app/mobile-twitter-verified-filter-view');
var MobilePositiveSightingFilterView = require('app/mobile-positive-sighting-filter-view');
var MobileOvalForecastLegendView = require('app/mobile-oval-forecast-legend-view');
var MobileOvalEstimateLegendView = require('app/mobile-oval-estimate-legend-view');
var MobileScientistLegendView = require('app/mobile-scientist-legend-view');
var Home = require('app/home');
var Dropdown = require('app/dropdown');
var MainMapView = require('app/main-map-view');

module.exports = Marionette.Controller.extend({
    initialize: function(options) {
        
    },

    initOvalFilters: function() {
        this.initOvalForecastView();
        this.initOvalEstimateView();
        this.initViewAndPrependWithModel(SightingHeatmapLayerView, '#estimates-container', Backbone.Model);
    },

    initWeatherFilters: function() {
        this.initViewAndAppendWithModel(CurrentConditionsView, '#weather-container', Backbone.Model);
        this.initViewAndAppendWithModel(CloudCoverView, '#weather-container', Backbone.Model);
        this.initViewAndAppendWithModel(DayNightLayerView, '#weather-container', Backbone.Model);
    },

    initLegend: function() {
        this.initViewAndAppendWithCollection(ScientistFilterView,        '#map-legend', ScientistCollection);
        this.initViewAndAppendWithCollection(PositiveSightingFilterView, '#map-legend', WebObservationCollection);
        this.initViewAndAppendWithCollection(NegativeSightingFilterView, '#map-legend', WebObservationCollection);
        this.initViewAndAppendWithCollection(TwitterUnverifiedFilterView,'#map-legend', TwitterObservationCollection);
        this.initViewAndAppendWithCollection(TwitterVerifiedFilterView,  '#map-legend', TwitterObservationCollection);
        this.initViewAndAppendWithCollection(OvalForecastLegendView,     '#map-legend', Backbone.Model);
        this.initViewAndAppendWithCollection(OvalEstimateLegendView,     '#map-legend', Backbone.Model);
    },

    initPopups: function(){
        this.initViewAndPrependWithModel(HelpPopupView, 'main', Backbone.Model);
    },

    initOvalForecastView: function() {
        var config = {
            $container: $('#map-canvas')
        };
        var map = new Map(config)
        map.render();

        var view = new OvalForecastView({
            model: new Backbone.Model()
        });

        $('#estimates-container').append(view.render().el);
    },

    initOvalEstimateView: function() {
        var config = {
            $container: $('#map-canvas')
        };
        var map = new Map(config)
        map.render();

        var view = new OvalEstimateView({
            model: new Backbone.Model()
        });

        $('#estimates-container').append(view.render().el);
    },

    initDateTimePicker: function() {
        new DateAndTimePickerView({
            model: new Backbone.Model()
        }).render();
    },

    /*----- Mobile Cards ------*/
    initMobileFilters: function(){
        //estimates
        this.initViewAndPrependWithModel(MobileOvalForecastView, '#mobile-estimates-container', Backbone.Model);
        this.initViewAndPrependWithModel(MobileOvalEstimateView, '#mobile-estimates-container', Backbone.Model);
        this.initViewAndPrependWithModel(MobileCitizenEstimateView, '#mobile-estimates-container', Backbone.Model);

        //weather
        this.initViewAndPrependWithModel(MobileCloudCoverView, '#mobile-weather-container', Backbone.Model);
        this.initViewAndPrependWithModel(MobileCurrentConditionsView, '#mobile-weather-container', Backbone.Model);
        this.initViewAndPrependWithModel(MobileDayNightLayerView, '#mobile-weather-container', Backbone.Model);
    },

    initMobileLegend: function(){
        //legends
        this.initViewAndAppendWithCollection(MobileScientistLegendView, '#mobile-map-legend', ScientistCollection);
        this.initViewAndAppendWithCollection(MobilePositiveSightingFilterView, '#mobile-map-legend', WebObservationCollection);
        this.initViewAndAppendWithCollection(MobileNegativeSightingFilterView, '#mobile-map-legend', WebObservationCollection);
        this.initViewAndAppendWithCollection(MobileTwitterUnverifiedFilterView,'#mobile-map-legend',TwitterObservationCollection);
        this.initViewAndAppendWithCollection(MobileTwitterVerifiedFilterView, '#mobile-map-legend', TwitterObservationCollection);
        this.initViewAndAppendWithCollection(MobileOvalForecastLegendView,    '#mobile-map-legend', Backbone.Model);
        this.initViewAndAppendWithCollection(MobileOvalEstimateLegendView,    '#mobile-map-legend', Backbone.Model);
    },

    /*----- Popups ------*/
    showAffirmativeReportPopupView: function(){
        var view = new AffirmativeReportPopupView({
            model: new WebObservation()
        });
        App.reportPopover.show(view);
    },

    showNegativeReportPopupView: function(){
        var view = new NegativeReportPopupView({
            model: new WebObservation()
        });
        App.reportPopover.show(view);
    },

    /*-- View factories --*/
    initViewAndAppendWithModel: function(Clazz, appendTo, model){
        var view = new Clazz({
            model: new model()
        });
        $(appendTo).append(view.render().el);
    },

    initViewAndPrependWithModel: function(Clazz, prependTo, model){
        var view = new Clazz({
            model: new model()
        });
        $(prependTo).prepend(view.render().el);
    },    

    initViewAndAppendWithCollection: function(Clazz, appendTo, collection){
        var view = new Clazz({
            collection: new collection()
        });
        $(appendTo).append(view.render().el);
    },

    initViewAndPrependWithCollection: function(Clazz, prependTo, collection){
        var view = new Clazz({
            collection: new collection()
        });
        $(prependTo).prepend(view.render().el);
    },

    showPage: function() {
        new Home();
        App.main.show(new MainMapView({model: new Backbone.Model()}));
        
        this.initOvalFilters();
        this.initWeatherFilters();
        this.initLegend();
 
        this.initMobileFilters();
        this.initMobileLegend();

        this.initDateTimePicker();
        this.initViewAndPrependWithModel(OtherMapControlsView, 'main', Backbone.Model);
        this.initViewAndPrependWithModel(MapDrawerLayout, 'main', Backbone.Model);
        
        //mobile view
        if (window.screen.width <= 360){
            this.initViewAndAppendWithModel(SightingControlsView, '#auroraReport', Backbone.Model);
            var auroraMenuItem = new Dropdown();
            auroraMenuItem.init($('#auroraReport'), $('#auroraReport > div'));
        }
        //tablet view
        else if (window.screen.width <= 768 && window.screen.width > 360){
            this.initViewAndPrependWithModel(SightingControlsView, '#navBar', Backbone.Model);
        }
        //desktop view
        else{
            this.initViewAndPrependWithModel(SightingControlsView, '#navBar > ul', Backbone.Model);
        }
        this.initHandlers();
        this.initPopups();

        App.addRegions({reportPopover: '#reportPopupContainer'});
    },

    /*-- Handlers --*/
    initHandlers: function() {
        App.vent.on('open-affirmative', this.showAffirmativeReportPopupView.bind(this));
        App.vent.on('open-negative', this.showNegativeReportPopupView.bind(this));
    }
});