'use strict';

var C = require('app/consts');

module.exports = {
	updateLocation: function(loc) {
        var deferred = $.Deferred();
        var country = "",
            state = "";
        var location = this._parseLocation(loc);
        var address = location.formatted_address;
        var latLng = new google.maps.LatLng(location.latitude, location.longitude);

        // use reverse lookup to find best-guess location of user.
        new google.maps.Geocoder().geocode( { 'latLng': latLng}, function(results, status) {
            if (status == google.maps.GeocoderStatus.OK) {
                // the formatted address
                var formatted_address = results[0].formatted_address;
                
                // find the state and country information
                var t_add = results[0].address_components;
                for (var i=0; i<t_add.length; i++) {
                    if (t_add[i].types[0] == 'administrative_area_level_1') {
                        state = t_add[i].short_name;
                    }
    
                    if ( _.contains(t_add[i].types, 'country') ) {
                        country = t_add[i].long_name;
                    }
                }

                var result = {
                    formatted_address: address || formatted_address,
                    address_state: state,
                    address_country: country,
                    location: { latitude: location.latitude, longitude: location.longitude }
                };

                deferred.resolve(result);
            }
        });

        return deferred;
    },

    locationTextSearch: function(location) {
    	return $.ajax('https://maps.googleapis.com/maps/api/place/textsearch/json?query='+ location +'&key=' + C.get('GOOGLE_API_KEY'));
    },

    _parseLocation: function(location) {
        if(location.longitude) {
            return {
                longitude: location.longitude,
                latitude: location.latitude,
                formatted_address: null
            };
        } else {
            var lng = location.geometry.location.lng.prototype ? location.geometry.location.lng() : location.geometry.location.lng;
            var lat = location.geometry.location.lat.prototype ? location.geometry.location.lat() : location.geometry.location.lat;
            return {
                longitude: lat,
                latitude: lng,
                formatted_address: location.formatted_address
            };
        }
    }
};