'use strict';

var User = require('app/user');
var UserProfile = require('app/user-profile');
var SocialAuthUser = require('app/social-auth-user');
var FinshRegistrationView = require('app/finish-registration-view');
var App = require('app/app');
var C = require('app/consts');

var googleapi = {
    setToken: function(data) {
        //Cache the token
        localStorage.access_token = data.access_token;
        //Cache the refresh token, if there is one
        localStorage.refresh_token = data.refresh_token || localStorage.refresh_token;
        //Figure out when the token will expire by using the current
        //time, plus the valid time (in seconds), minus a 1 minute buffer
        var expiresAt = new Date().getTime() + parseInt(data.expires_in, 10) * 1000 - 60000;
        localStorage.expires_at = expiresAt;
    },
    authorize: function(options) {
        var deferred = $.Deferred();

        //Build the OAuth consent page URL
        var authUrl = 'https://accounts.google.com/o/oauth2/auth?' + $.param({
            client_id: options.client_id,
            redirect_uri: options.redirect_uri,
            response_type: 'code',
            scope: options.scope
        });

        //Open the OAuth consent page in the InAppBrowser
        var authWindow = window.open(authUrl, '_blank', 'location=no,toolbar=no');

        //The recommendation is to use the redirect_uri "urn:ietf:wg:oauth:2.0:oob"
        //which sets the authorization code in the browser's title. However, we can't
        //access the title of the InAppBrowser.
        //
        //Instead, we pass a bogus redirect_uri of "http://localhost", which means the
        //authorization code will get set in the url. We can access the url in the
        //loadstart and loadstop events. So if we bind the loadstart event, we can
        //find the authorization code and close the InAppBrowser after the user
        //has granted us access to their data.
        authWindow.addEventListener('loadstart', googleCallback);
        function googleCallback(e){
            var url = (typeof e.url !== 'undefined' ? e.url : e.originalEvent.url);
            var code = /\?code=(.+)$/.exec(url);
            var error = /\?error=(.+)$/.exec(url);

            if (code || error) {
                //Always close the browser when match is found
                authWindow.close();
            }

            if (code) {
                //Exchange the authorization code for an access token
                $.post('https://accounts.google.com/o/oauth2/token', {
                    code: code[1],
                    client_id: options.client_id,
                    client_secret: options.client_secret,
                    redirect_uri: options.redirect_uri,
                    grant_type: 'authorization_code'
                }).done(function(data) {
                    googleapi.setToken(data);
                    data.code = code[1];
                    deferred.resolve(data);
                }).fail(function(response) {
                    deferred.reject(response.responseJSON);
                });
            } else if (error) {
                //The user denied access to the app
                deferred.reject({
                    error: error[1]
                });
            }
        }

        return deferred.promise();
    },
    getToken: function(options) {
        var deferred = $.Deferred();

        if (new Date().getTime() < localStorage.expires_at) {
            deferred.resolve({
                access_token: localStorage.access_token
            });
        } else if (localStorage.refresh_token) {
            $.post('https://accounts.google.com/o/oauth2/token', {
                refresh_token: localStorage.refresh_token,
                client_id: options.client_id,
                client_secret: options.client_secret,
                grant_type: 'refresh_token'
            }).done(function(data) {
                googleapi.setToken(data);
                deferred.resolve(data);
            }).fail(function(response) {
                deferred.reject(response.responseJSON);
            });
        } else {
            deferred.reject();
        }

        return deferred.promise();
    },
    userInfo: function(options) {
        return $.getJSON('https://www.googleapis.com/oauth2/v1/userinfo', options);
    }
};


module.exports = Marionette.ItemView.extend({
    className: '',

    template: Handlebars.compile(
        '<div class="row pageTitle">'+
            '<h2 class="large-12 small-12 medium-12 columns">{{#ifIsMobile}}<span id="mobileBackButton"><img src="img/NavBackIcon.png"></span>{{/ifIsMobile}}REGISTER/SIGN IN</h2>'+
        '</div>'+
        '<div class="row">'+
            '<div class="large-12 small-12 medium-12">'+
                '<p>Please create a profile via your preferred social login and to be notified of real observations of aurora, '+
                'include your email address. By logging in, you agree to our <a id="privacyPolicy">privacy policy<a> which is aimed at strictly '+
                'protecting your privacy while enabling scientific research about the aurora and use of this application. '+
                'To be included in the Points Leaderboard, please include a Username. Thanks!</p>'+
            '</div>'+
        '</div>'+
        'div class="login-btn-group">'+
            '<div class="row">'+
                '<div class="medium-12 small-12 columns">'+
                    '<button id="facebook" class="small-10">Facebook</button>'+
                '</div>'+
            '</div>'+
            '<div class="row">'+
                '<div class="medium-12 small-12 columns">'+
                    '<button id="googlePlus" class="small-10">Google+</button>'+
                '</div>'+
            '</div>'+
            '<div class="row">'+
                '<div class="medium-12 small-12 columns">'+
                    '<button id="twitter" class="small-10">Twitter</button>'+
                '</div>'+
            '</div>'+
        '</div>'
        ),

    privacyPolicyTemplate: aur.tpl['privacy-policy-tpl'],

    ui: {
        facebookBtn: '#facebook',
        twitterBtn: '#twitter',
        googleBtn: '#googlePlus',
        privacyPolicy: '#privacyPolicy'
    },

    events: {
        'click @ui.facebookBtn': 'loginFB',
        'click @ui.twitterBtn': 'loginTwtr',
        'click @ui.googleBtn': 'loginGoogle',
        'click @ui.privacyPolicy': 'showPrivacyPolicy'
    },

    showPrivacyPolicy: function() {
        navigator.notification.alert(this.privacyPolicyTemplate(), function(){}, 'Privacy Policy');
    },

    loginFB: function() {
        FacebookInAppBrowser.settings.appId = '213963942137216';
        FacebookInAppBrowser.settings.redirectUrl = 'http://localhost:8000/settings';
        FacebookInAppBrowser.settings.permissions = 'public_profile, email';

        FacebookInAppBrowser.login({
            send: function() {
                console.log('login opened');
            },
            success: _.bind(function(accessToken, expiresIn) {
                console.log('window closed');
                if(accessToken) {
                    this.accessToken = accessToken;
                    console.log(accessToken);
                } else {
                    console.log('no access token');
                }

                if(expiresIn) {
                    this.expiresIn = expiresIn;
                    console.log(expiresIn);
                } else {
                    console.log('no expires in value');
                }
            }, this),
            denied: function() {
                console.log('user denied');
            },
            timeout: function(){
                console.log('a timeout has occurred, probably a bad internet connection');
            },
            complete: _.bind(function(accessToken) {
                console.log('Complete ', accessToken);
            },this),
            userInfo: _.bind(function(userInfo) {
                var data = {
                    provider: 'facebook',
                    uid: userInfo.id,
                    access_token: this.accessToken
                };

                $.ajax({
                    url: C.get('host') + '/social-login', 
                    data: data
                })
                .done(_.bind(function(data) {
                    App.commands.execute('loginUser', data);
                    App.execute('routeSettings');
                }, this))
                .fail(_.bind(function() {
                    this._createFbUser(userInfo);
                }, this));
            }, this)
        });
    },

    _createFbUser: function(userInfo) {
        if(userInfo) {
            var accessToken = this.accessToken;
            var user = new User();
            var socialAuthUser = new SocialAuthUser();
            var userProfile = new UserProfile();
            var extraData = {};

            extraData.access_token = this.accessToken;
            extraData.expires = this.expiresIn;
            extraData.uid = userInfo.id;

            socialAuthUser.set('extra_data', JSON.stringify(extraData));
            socialAuthUser.set('provider', 'facebook');
            socialAuthUser.set('uid', userInfo.id);

            user.set('first_name', userInfo.first_name);
            user.set('last_name', userInfo.last_name);
            user.set('email', userInfo.email);
            user.set('username', userInfo.username);

            userProfile.set('social_auth_user', socialAuthUser.toJSON());
            userProfile.set('user', user.toJSON());
            userProfile.urlRoot = C.get('host') + '/user';

            var registrationView = new FinshRegistrationView({
                model: userProfile
            });
            App.main.show(registrationView);
        } else {
            console.log('no user info');
        }
    },

    loginTwtr: function() {
        var self = this;
        var options = { 
            consumerKey: 'YfTD3lQrUHwANPZTWgoSjw',
            consumerSecret: 'tBUGEXulfFqryJV69Be6CXtQOMeBfMOgZqBho5u54',
            callbackUrl: C.get('host') + '/settings' };

        var oauth = OAuth(options);
        oauth.get('https://api.twitter.com/oauth/request_token',
            function(data) {
                var requestParams = data.text;
                console.log('Getting auth', data);
                var twitterView = window.open('https://api.twitter.com/oauth/authenticate?'+data.text, '_blank', 'location=no');
                twitterView.addEventListener('loadstart', function(location) {
                    console.log('location', location);
                    if (location === 'http://localhost:8000/settings?denied') {
                        twitterView.close();
                    } else if(location.url.search('oauth_verifier=') >= 0) {
                        var oauth_verifier = location.url.match(/oauth_verifier=.*/)[0].substring(15);
                        twitterView.close();

                        oauth.get('https://api.twitter.com/oauth/access_token?oauth_verifier='+oauth_verifier+'&'+requestParams,
                            function(data) {
                                var params = data.text.split('&');
                                var result = {};

                                // Set "oauth_token", "oauth_token_secret", screen_name, etc. on result object
                                _.each(params, function(param) {
                                    var arrayKeyVal = param.split('=');
                                    result[arrayKeyVal[0]] = arrayKeyVal[1]; 
                                });
                                result.provider = 'twitter';
                                result.uid = result.user_id;

                                $.ajax({
                                    url: C.get('host') + '/social-login', 
                                    data: result
                                })
                                .done(_.bind(function(data) {
                                    App.commands.execute('loginUser', data)
                                    App.execute('routeSettings');
                                }, this))
                                .fail(_.bind(function() {
                                    self._createTwitterUser.call(self, result);
                                }, this));

                            }, function(data) {
                                console.log('Oauth failure', data);
                            }
                        );
                    }
                });
            },
            function(data) { 
                console.log('No Auth', data);
            }
        );
    },

    _createTwitterUser: function(userInfo) {
        if(userInfo) {
            var user = new User(),
                socialAuthUser = new SocialAuthUser(),
                userProfile = new UserProfile(),
                extraData = {access_token: {}};

            extraData.access_token.oauth_token = userInfo.oauth_token;
            extraData.access_token.oauth_token_secret = userInfo.oauth_token_secret;
            extraData.access_token.screen_name = userInfo.screen_name;
            extraData.access_token.user_id = userInfo.user_id;
            extraData.id = userInfo.user_id;

            socialAuthUser.set('extra_data', JSON.stringify(extraData));
            socialAuthUser.set('provider', 'twitter');
            socialAuthUser.set('uid', userInfo.user_id);

            user.set('username', userInfo.screen_name);

            userProfile.set('social_auth_user', socialAuthUser.toJSON());
            userProfile.set('user', user.toJSON());
            userProfile.urlRoot = C.get('host') + '/user';

            var registrationView = new FinshRegistrationView({
                model: userProfile
            });
            App.main.show(registrationView);
        } else {
            console.log('no user info');
        }
    },

    loginGoogle: function() {
        var self = this;
        googleapi.authorize({
            client_id: C.get('google_client_id'),
            client_secret: C.get('google_client_secret'),
            redirect_uri: 'http://aurorasaurus.ideum.com/settings',
            scope: C.get('google_scope')
        }).done(function(data) {
            googleapi.getToken({
                client_id: C.get('google_client_id'),
                client_secret: C.get('google_client_secret'),
            }).then(function(data) {
                //Pass the token to the API call and return a new promise object
                return googleapi.userInfo({ access_token: data.access_token });
            }).done(function(user) {
                var data = {
                    provider: 'google-plus',
                    uid: user.email,
                    access_token: window.localStorage.access_token,
                };

                $.ajax({
                    url: C.get('host') + '/social-login', 
                    data: data
                })
                .done(function(data) {
                    App.commands.execute('loginUser', data);
                    App.execute('routeSettings');
                })
                .fail(function() {
                    self._createGoogleUser.call(self, user);
                });
            }).fail(function() {
                //If getting the token fails, or the token has been
                //revoked, show the login view.
                console.log('Failed');
            });
        }).fail(function(data) {
            console.log(data.error);
        });
    },

    _createGoogleUser: function(userInfo) {
        if(userInfo) {
            var accessToken = this.accessToken,
                user = new User(),
                socialAuthUser = new SocialAuthUser(),
                userProfile = new UserProfile(),
                extraData = {};

            extraData.access_token = window.localStorage.access_token;
            extraData.expires = window.localStorage.expiresAt;
            extraData.user_id = userInfo.id;
            extraData.code = null;

            socialAuthUser.set('extra_data', JSON.stringify(extraData));
            socialAuthUser.set('provider', 'google-plus');
            socialAuthUser.set('uid', userInfo.email);

            user.set('first_name', userInfo.given_name);
            user.set('last_name', userInfo.family_name);
            user.set('username', userInfo.email);

            userProfile.set('social_auth_user', socialAuthUser.toJSON());
            userProfile.set('user', user.toJSON());
            userProfile.urlRoot = C.get('host') + '/user';

            var registrationView = new FinshRegistrationView({
                model: userProfile
            });
            App.main.show(registrationView);
        } else {
            console.log('no user info');
        }
    }


});