'use strict';

var App = require('app/app');
var TwitterVote = require('app/twitter-vote');
var TweetView = require('app/tweet-view');
var TweetVoteView = require('app/tweet-vote-view');
var VerifiedTweetVoteView = require('app/verified-tweet-vote-view');
var WebObservation = require('app/web-observation');
var WebSightingDetailView = require('app/web-sighting-detail-view');
var WebSightingClassificationView = require('app/web-sighting-classification-view');
var Scientist = require('app/scientist');
var ScientistDrawerView = require('app/scientist-drawer-view');

module.exports = Marionette.LayoutView.extend({

	id: 'mapDrawer',
    template: aur.tpl['twitter-card-view-tpl'],

    regions: {
    	top: '#tweetView', 
    	bottom: '#voteFormView'
    },

    ui: { 
		openButton: '#mapDrawerActivateButton'
    },

    events: {
    	'click @ui.openButton': 'toggleMapDrawer'
    },

    initialize: function() {
    	this.initHandlers();
    },

    initHandlers: function() {
        App.vent.on('unverifiedTwitterMarkerClicked', this.handleUnverifiedTwitter.bind(this));
        App.vent.on('verifiedTwitterMarkerClicked', this.handleVerifiedTwitter.bind(this));
        App.vent.on('positiveWebSightingMarkerClicked', this.handlePositiveWebSighting.bind(this));
        App.vent.on('negativeWebSightingMarkerClicked', this.handleNegativeWebSighting.bind(this));
        App.vent.on('scientistMarkerClicked', this.handleScientist.bind(this));
    },

    toggleMapDrawer: function() {
        this.$el.toggleClass('open');
    },

    handleMarkerClicked: function(content){
    	if(this.$el.hasClass('open')){
            return;
        }
        
        this.openDrawer(content);
    },

    handleUnverifiedTwitter: function(content) {
        var modelContent = new TwitterVote();
        modelContent.set('tweet', content);

        this.top.show(new TweetView({model: new Backbone.Model(content)}));
        this.bottom.show(new TweetVoteView({model: modelContent}));
        this.handleMarkerClicked();
    },

    handleVerifiedTwitter: function(content) {
        var modelContent = new TwitterVote();
        modelContent.set('tweet', content);

        this.top.show(new TweetView({model: new Backbone.Model(content)}));
        this.bottom.show(new VerifiedTweetVoteView({model: new TwitterVote({observation: content})}));
        this.handleMarkerClicked();
    },

    handlePositiveWebSighting: function(content) {
        var observation = new WebObservation(content);

        this.top.show(new WebSightingDetailView({model: observation}));
        this.bottom.show(new WebSightingClassificationView({model: observation}));
        this.handleMarkerClicked();
    },

    handleNegativeWebSighting: function(content) {
        var observation = new WebObservation(content);

        this.top.show(new WebSightingDetailView({model: observation}));
        this.bottom.show(new WebSightingClassificationView({model: observation}));
        this.handleMarkerClicked();
    },

    handleScientist: function(content) {
        var scientist = new Scientist(content);

        this.top.show(new ScientistDrawerView({model: scientist}));
        this.bottom.reset();
        this.handleMarkerClicked();
    },

    openDrawer: function(){
        if(!this.$el.hasClass('open')){
            this.$el.addClass('visible');
            this.$el.addClass('open');
            this.ui.openButton.addClass('open');
        }
    },

    closeDrawer: function(){
        this.$el.removeClass('open');
        this.ui.openButton.removeClass('open');
    }
});