'use strict';

var App = require('app/app');
var map;
    
var Map = function(config) {
    config = config || {};
    this.detectLocation = true;
    this.mapOptions = config.mapOptions;
    this.$container = config.$container;
    this.map = map;
    this.initHandlers();
};

Map.prototype.render = function() {
    if (this.map){
        return;
    }

    var detectLocation = this.detectLocation && navigator.geolocation;
    var mapOptions = this.mapOptions || {
        zoom: 6,
        maxZoom: 12,
        minZoom: 2
    };

    mapOptions.mapTypeControlOptions = {
        position: google.maps.ControlPosition.LEFT_TOP
    };

    mapOptions.zoomControlOptions = {
        position: google.maps.ControlPosition.RIGHT_CENTER
    };

    mapOptions.panControlOptions = {
        position: google.maps.ControlPosition.RIGHT_BOTTOM
    },

    mapOptions.streetViewControl = false,

    this.map = new google.maps.Map(this.$container[0], mapOptions);
    map = this.map

    this.handleNoGeolocation();

    // Try HTML5 geolocation
    if (detectLocation) {
        navigator.geolocation.getCurrentPosition(function(position) {
            var pos = new google.maps.LatLng(position.coords.latitude, position.coords.longitude);
            App.reqres.setHandler('userCoords', function() { return position; });

            this.map.setCenter(pos);
        }.bind(this), function() {
            this.handleNoGeolocation();
        }.bind(this));
    } else {
        this.handleNoGeolocation();
    }
};

Map.prototype.handleNoGeolocation = function() {
    this.map.setCenter(new google.maps.LatLng(65, -84));
    this.map.setZoom(3);
};

Map.prototype.initHandlers = function() {
        App.commands.setHandler('resizeMap', _.bind(function() {
            var tries = 0;
            var interval = setInterval(_.bind(function() {
                google.maps.event.trigger(this.map, 'resize');
                if(tries === 5) clearInterval(interval);
            }, this), 2000);
        }, this));
};

Map.prototype.getMap = function() {
    return this.map
};

Map.destroy = function() {
    map = null;
};

module.exports = Map;