'use strict';

var App = require('app/app');
var C = require('app/consts');
var MobilePhotoService = require('app/mobile-photo-service');
var WebObservation = require('app/web-observation');
var LocationService = require('app/location-service');

var MobileBase = function() {
    $.ajax(C.get('host') + '/token')
    .done(function(data) {
       $.ajaxSetup({
            headers: {
                'X-CSRFToken': data
            }
        }); 
    });
	document.addEventListener('deviceready', _.bind(function() {
        window.fbAsyncInit = function() {
            window.FB.init({
              appId      : '213963942137216',
              xfbml      : true,
              version    : 'v2.0'
            });
          };

          (function(d, s, id){
             var js, fjs = d.getElementsByTagName(s)[0];
             if (d.getElementById(id)) {return;}
             js = d.createElement(s); js.id = id;
             js.src = "//connect.facebook.net/en_US/sdk.js";
             fjs.parentNode.insertBefore(js, fjs);
           }(document, 'script', 'facebook-jssdk'));
		this.onReady();
	}, this));
};

MobileBase.prototype.onReady = function() {
    window.analytics.startTrackerWithId('UA-55108991-4');
    StatusBar.overlaysWebView(false);
    wizSpinner.create();


    $( document ).ajaxStart(function() {
        wizSpinner.show();
        // spinnerplugin.show();
    });
    $( document ).ajaxStop(function() {
        wizSpinner.hide();
        // spinnerplugin.hide();
    });

    App.reqres.setHandler('isConnected', _.bind(function() {
        return this.isConnected();
    }, this));

    if(this.isConnected()) this.sendSavedWebObservations();

    document.addEventListener("offline", _.bind(this.setOfflineHandler,this), false);
    document.addEventListener("online", _.bind(this.setOnlineHandler, this), false);
};

MobileBase.prototype.setOfflineHandler = function() {
	
};

MobileBase.prototype.setOnlineHandler = function() {
    // this._fetchScripts().done(_.bind(function() {
        this.sendSavedWebObservations();
    // }, this));
};

MobileBase.prototype.isConnected = function() {
    var networkState = navigator.network.connection.type;

    var states = {};
    states[Connection.UNKNOWN]  = 'Unknown connection';
    states[Connection.ETHERNET] = 'Ethernet connection';
    states[Connection.WIFI]     = 'WiFi connection';
    states[Connection.CELL_2G]  = 'Cell 2G connection';
    states[Connection.CELL_3G]  = 'Cell 3G connection';
    states[Connection.CELL_4G]  = 'Cell 4G connection';
    states[Connection.NONE]     = 'No network connection';

    if (states[networkState] === 'No network connection') {
        return false;
    } else {
        return true;
    }
};

MobileBase.prototype.sendSavedWebObservations = function() {
    var pending = window.localStorage.observationsPendingSave;
    var deferreds = [];

    if(!pending) pending = [];
    else pending = JSON.parse(pending);

    deferreds = this.saveAllWebObservations(pending);

    $.when.apply(this, deferreds)
    .done(function() {
        window.localStorage.removeItem('observationsPendingSave');
    });

};

MobileBase.prototype.saveAllWebObservations = function(pending) {
    var deferreds = [];

    _.each(pending, function(p) {
        var webObservation = new WebObservation(p.json);
        var $photoDef = $.Deferred().resolve();
        deferreds.push($photoDef);

        if(p.imageToUpload && p.imageToUpload.imageData) {
            $photoDef = MobilePhotoService.saveMobilePhoto(p.imageToUpload.imageData, p.imageToUpload.imageOptions);
        }

        deferreds.push(LocationService.locationTextSearch(p.location)
        .done(function(result) {
            deferreds.push(LocationService.updateLocation(result.results[0])
            .done(function(data) {

                webObservation.set(data);

                deferreds.push($photoDef.done(function(data) {
                    webObservation.set('imageUrl', data.save_url);
                    webObservation.save();
                }));
            }));
        }));
    });

    return deferreds;
};

MobileBase.prototype._fetchScripts = function() {
    var deferred = $.Deferred();
    if(typeof google !== 'undefined') {
        deferred.resolve();
    } else if(this.isConnected()) {
        $.getScript('http://maps.googleapis.com/maps/api/js?key=AIzaSyDc8l8XRXNKdT3WleJIJmwAkNgO0HZfgwg&sensor=true&libraries=weather,places,visualization')
        .done(function() {
            $.getScript(C.get('host') + '/static/js/vendor/daynightoverlay.js')
            .done(function() {
                deferred.resolve();
            });
        });
    } else {
        deferred.reject();
    }

    return deferred;
};

module.exports = MobileBase;