'use strict';

var App = require('app/app');
var Dropdown = require('app/dropdown');
var AffirmativeReportPopupView = require('app/affirmative-report-popup-view');
var NegativeReportPopupView = require('app/negative-report-popup-view');
var WebObservation = require('app/web-observation');

module.exports = Marionette.ItemView.extend({
	template: aur.tpl['mobile-header-view-tpl'],

	ui: {
		verifyTweetsNav: '#verify-tweets-nav',
		reportSightingNav: '#report-sighting-nav',
		spaceDataNav: '#space-data-nav',
		mapNav: '#map-nav',
		profileNav: '#profile-nav',
		landingNav: '#landing-nav',
		leaderboardNav: '#leaderboard-nav',
		homeNav: '.mobile-logo',
		sawAuroraBtn: '#yesButton',
		notSeeAuroraBtn: '#noButton',

		mobileSightingButton: '#auroraReport',
		mobileNotificationsButton: '#notificationItem',
		mobileMenuButton: '.navBarIcon'
	},

	events: {
		'click @ui.verifyTweetsNav': 'navVerifyTweets',
		'click @ui.reportSightingNav': 'navReportSighting',
		'click @ui.spaceDataNav': 'navSpaceData',
		'click @ui.mapNav': 'navMap',
		'click @ui.profileNav': 'navProfile',
		'click @ui.landingNav': 'navLanding',
		'click @ui.leaderboardNav': 'navLeaderboard',
		'click @ui.homeNav': 'navLanding',
		'click @ui.sawAuroraBtn': 'handleSawAurora',
		'click @ui.notSeeAuroraBtn': 'handleNotSeeAurora'
	},

	onRender: function(){
		var reportDropdown = new Dropdown();
		reportDropdown.init(this.ui.mobileSightingButton, this.$('#auroraReport > div'));

		var notificationDropdown = new Dropdown();
		notificationDropdown.init(this.ui.mobileNotificationsButton, this.$('#notificationContainer'));

		var menuDropdown = new Dropdown();
		menuDropdown.init(this.ui.mobileMenuButton, this.$('#navBar > ul'));
	},

	navVerifyTweets: function() {
		App.execute('routeVerifyTweets');
	},

	navReportSighting: function() {
		App.execute('routeReportASighting');
	},

	navSpaceData: function() {
		App.execute('routeSpaceData');
	},

	navMap: function() {
		App.execute('routeAuroraMap');
	},

	navProfile: function() {
		App.execute('routeSettings');	
	},

	navLanding: function() {
		App.execute('routeLanding');
	},

	navLeaderboard: function() {
		App.execute('routeLeaderboard');	
	},

	handleSawAurora: function() {
		var view = new AffirmativeReportPopupView({
            model: new WebObservation()
        });
        App.main.show(view);
	},

	handleNotSeeAurora: function() {
		var view = new NegativeReportPopupView({
            model: new WebObservation()
        });
        App.main.show(view);
	}
});