'use strict';

var App = require('app/app');
var Utils = require('app/utils');
var C = require('app/consts');

module.exports = Marionette.ItemView.extend({
    id: 'otherMapControls',

    template: Handlebars.compile(
        '<span class="blackBg" style="display:none;">' +
            'Did you see the aurora?' +
            '<input id="yesButton" class="affirmativeButton" type="button" value="Yes">' +
          '<input id="noButton" class="negativeButton" type="button" value="No">' +
        '</span>' +
        '<img id="fullscreenButton" src="/static/img/fullscreen.png"></div>'+
        '<div class="mapDateTimeControls">'+
            '<input id="map-start-time" value="{{fmtTime start_date}}" type="text">'+
            '<input id="map-start-date" value="{{fmtDate start_date}}" type="text">'+
        '</div>'
    ),

    ui: {
        fullscreenButton: '#fullscreenButton',
        buttons: '.blackBg',
        yesButton: '#yesButton',
        noButton: '#noButton',
        startDate: '#map-start-date',
        startTime: '#map-start-time'
    },

    events: {
        'click @ui.fullscreenButton': 'toggleFullscreen',
        'click @ui.yesButton': 'toggleAffirmativeReportPopup',
        'click @ui.noButton': 'toggleNegativeReportPopup'
    },

    initialize: function() {
        this.model.set('start_date', Utils.round15Minutes(moment()).format(Utils.defaultDateFmtWithTZ()));
    },

    onRender: function() {
        this.renderDateTimePickers();
    },

    renderDateTimePickers: function(){
        var date = Utils.round15Minutes(moment()).format(Utils.defaultDateFmtWithTZ())
        var self = this;
        var timeOptions = { 
            interval: 15,
            clear: false,
            onSet: function() { self.updateDateOrTime.call(self, 'time', this) }
        };
        var dateOptions = { 
            max: new Date(),
            format: 'd mmm yyyy',
            clear: false,
            onSet: function() { self.updateDateOrTime.call(self, 'date', this) }
        };

        this.ui.startDate.pickadate(dateOptions);
        this.ui.startTime.pickatime(timeOptions);
    },

    updateDateOrTime: function(typeToUpdate, pickerObj) {
            var newTime = pickerObj.get('select');
            var oldTime = this.model.get('start_date');
            var oldTimeMoment = moment(oldTime);

            if(typeToUpdate === 'date') {
                oldTimeMoment.year(newTime.year);
                oldTimeMoment.month(newTime.month);
                oldTimeMoment.date(newTime.date);
            } else {
                oldTimeMoment.hours(newTime.hour);
                oldTimeMoment.minutes(newTime.mins);
            }

            this.model.set('start_date', oldTimeMoment.format(Utils.defaultDateFmtWithTZ()));
            App.vent.trigger('mapDateChanged', this.model.get('start_date'));
        },

    toggleAffirmativeReportPopup: function(){
        App.vent.trigger('open-affirmative');
    },

    toggleNegativeReportPopup: function(){
        App.vent.trigger('open-negative');
    },

    toggleFullscreen: function() {
        if (!document.fullscreenElement &&    // alternative standard method
            !document.mozFullScreenElement && !document.webkitFullscreenElement && !document.msFullscreenElement ) {  // current working methods
            window.onresize = function(){return};
            if (document.documentElement.requestFullscreen) {
                document.documentElement.requestFullscreen();
            } else if (document.documentElement.msRequestFullscreen) {
                document.documentElement.msRequestFullscreen();
            } else if (document.documentElement.mozRequestFullScreen) {
                document.documentElement.mozRequestFullScreen();
            } else if (document.documentElement.webkitRequestFullscreen) {
                document.documentElement.webkitRequestFullscreen(Element.ALLOW_KEYBOARD_INPUT);
            }
            $('footer, header').css('display', 'none');
            Utils.mapResize(true);
            this.ui.fullscreenButton.attr('src', C.get('host') + '/static/img/close-fullscreen.png');
            this.ui.buttons.show();
        } 
        else {
            $('footer, header').css('display', 'block');
            if (document.exitFullscreen) {
                document.exitFullscreen();
            } else if (document.msExitFullscreen) {
                document.msExitFullscreen();
            } else if (document.mozCancelFullScreen) {
                document.mozCancelFullScreen();
            } else if (document.webkitExitFullscreen) {
                document.webkitExitFullscreen();
            }
            this.ui.fullscreenButton.attr('src', C.get('host') + '/static/img/fullscreen.png');
            this.ui.buttons.hide();
            window.onresize = function(){Utils.mapResize(false);};
            Utils.mapResize(false);
            setTimeout(function(){
                Utils.mapResize(false);
            }, 2000);
        }
    }
});