'use strict';

var C = require('app/consts');
var App = require('app/app');
var Map = require('app/map');
var BaseMapControlView = require('app/base-map-control-view');
var GeoJSON = require('app/geo-json');

var OvalEstimateView = Marionette.ItemView.extend({
	template: Handlebars.compile('<span id="oval-estimate">Oval Estimate</span>'),

    updateMessage: 'updateNowcastToggles',
    isActiveMessage: 'isOvalNowcastActive',
    toggleMessage: 'toggleNowcast',

	initialize: function(config) {
		this.FIFTEEN_MIN = 15*60*1000;
		this.nowcast_polygons = [];
        this.map = new Map().getMap();

        this.onInitialize();
	},

    initHandlers: function() {
        App.vent.on('mapDateChanged', function(dateString) {
            this.currentMoment = moment(dateString);
            if(this.isActive()) this.getLayerData();
        }.bind(this));
    },

    onRender: function() {
        this.$el.click(this.toggleLayer.bind(this));
    },

	getLayerData: function() {
            // send ajax call to get nowcast
            var int_time = this._getTime()
            $.ajax(C.get('host') + '/estimates?end_date=' + encodeURIComponent(int_time) +
                    '&local_offset=' + this._getLocalOffset() + '&forecast_lead=0'
            ).done(function(data, status, xhr) {
                this.showLayer(data, status, xhr);
            }.bind(this));
    },

    showLayer: function(data, status, xhr) {
        // after get a response from server, display nowcast polygons
        // first remove previous nowcast polygons if there is any
        this.removeLayer();

        var mapOptions = {
            strokeColor: "pink", //light purple
            strokeWeight: 0,
            strokeOpacity: 0.55,
            fillColor: "pink",
            fillOpacity: 0.55,
        };

        this.nowcast_polygons = this.showCast(this.nowcast_polygons, data[0].contour, mapOptions);
        this.el.classList.add('active');
        App.vent.trigger('updateNowcastToggles', true);
    },

    removeLayer: function() {
        // remove nowcast polygons on map 
        for (var i=0; i<this.nowcast_polygons.length; i++) {
            this.nowcast_polygons[i].setMap(null);
        }
        this.nowcast_polygons = new Array();
        this.el.classList.remove('active');
        App.vent.trigger(this.updateMessage, false);
    },

    showCast: function(polygons, contour, mapOptions) {
            var geoJson = new GeoJSON(contour, mapOptions);
            polygons = geoJson.geometryToGoogleMaps();
            for (var i = 0; i< polygons.length; i++) {
                polygons[i].setMap(this.map);
            }
            this.el.classList.add('active');
            return polygons;
    }
});

_.extend(OvalEstimateView.prototype, BaseMapControlView);

module.exports = OvalEstimateView;