'use strict';

var App = require('app/app');
var BaseMapControlView = require('app/base-map-control-view');
var Map = require('app/map');
var GeoJSON = require('app/geo-json');
var C = require('app/consts');

var OvalForecastView = Marionette.ItemView.extend({
	template: Handlebars.compile('<span id="oval-forecast">1 Hour Forecast</span>'),

    updateMessage: 'updateForecastToggles',
    isActiveMessage: 'isOvalForecastActive',
    toggleMessage: 'toggleForecast',

    initialize: function(config) {
        this.FIFTEEN_MIN = 15*60*1000;
        this.forecast_polygons = [];
        this.map = new Map().getMap();

        this.onInitialize();
    },

    initHandlers: function() {
        App.vent.on('mapDateChanged', function(dateString) {
            this.currentMoment = moment(dateString);
            if(this.isActive()) this.getLayerData();
        }.bind(this));
    },

    onRender: function() {
        this.$el.click(this.toggleLayer.bind(this));
    },

    removeLayer: function() {
        // remove forecast polygons on map 
        for (var i=0; i<this.forecast_polygons.length; i++) {
            this.forecast_polygons[i].setMap(null);
        }
        this.forecast_polygons = new Array();
        this.el.classList.remove('active');
        App.vent.trigger(this.updateMessage, false);
    },

    getLayerData: function() {
        var int_time = this._getTime()
        $.ajax(C.get('host') + '/estimates?end_date=' + encodeURIComponent(int_time) +
                    '&local_offset=' + this._getLocalOffset() + '&min_forecast_lead=0&max_forecast_lead=5000'
        ).done(function(data, status, xhr) {
            this.showLayer(data, status, xhr)
        }.bind(this));
    },

    showLayer: function(data, status, xhr) {
        // after get a response from server, display forecast polygons
        // first remove previous forecast polygons if there is any
        this.removeLayer();

        if (data==null || data.error == true) {
            forecast = -1;
            alert('Error');
        } else {
            var mapOptions = {
                strokeColor: "#9E78EF",
                strokeWeight: 0,
                strokeOpacity: 0.55,
                fillColor: "#9E78EF",
                fillOpacity: 0.55
            };
            
            this.forecast_polygons = this.showCast(this.forecast_polygons, data[0].contour, mapOptions)
        }
        this.el.classList.add('active');
        App.vent.trigger(this.updateMessage, true);
    },


    showCast: function(polygons, contour, mapOptions) {
            var geoJson = new GeoJSON(contour, mapOptions);
            polygons = geoJson.geometryToGoogleMaps();
            for (var i = 0; i< polygons.length; i++) {
                polygons[i].setMap(this.map);
            }
            return polygons;
    },

    // DEPRECATED 
    //tooltip for polygons
    tooltip: function(){
    	var id = 'tt';
    	var top = 3;
    	var left = 3;
    	var maxw = 300;
    	var speed = 10;
    	var timer = 20;
    	var endalpha = 95;
    	var alpha = 0;
    	var tt,t,c,b,h;
    	var ie = document.all ? true : false;
    	return{
    		show:function(v,w){
    			if(tt == null){
    				tt = document.createElement('div');
    				tt.setAttribute('id',id);
    				t = document.createElement('div');
    				t.setAttribute('id',id + 'top');
    				c = document.createElement('div');
    				c.setAttribute('id',id + 'cont');
    				b = document.createElement('div');
    				b.setAttribute('id',id + 'bot');
    				tt.appendChild(t);
    				tt.appendChild(c);
    				tt.appendChild(b);
    				document.body.appendChild(tt);
    				tt.style.opacity = 0;
    				tt.style.filter = 'alpha(opacity=0)';
    				document.onmousemove = this.pos;
    			}
    			tt.style.display = 'block';
    			c.innerHTML = v;
    			tt.style.width = w ? w + 'px' : 'auto';
    			tt.style.position = "absolute";
    			if(!w && ie){
    				t.style.display = 'none';
    				b.style.display = 'none';
    				tt.style.width = tt.offsetWidth;
    				t.style.display = 'block';
    				b.style.display = 'block';
    			}
    			if(tt.offsetWidth > maxw){tt.style.width = maxw + 'px'}
    			h = parseInt(tt.offsetHeight) + top;
    			clearInterval(tt.timer);
    			tt.timer = setInterval(function(){tooltip.fade(1)},timer);
    		},
    		pos:function(e){
    			var u = ie ? event.clientY + document.documentElement.scrollTop : e.pageY;
    			var l = ie ? event.clientX + document.documentElement.scrollLeft : e.pageX;
    			tt.style.top = (u - h) + 'px';
    			tt.style.left = (l + left) + 'px';
    		},
    		fade:function(d){
    			var a = alpha;
    			if((a != endalpha && d == 1) || (a != 0 && d == -1)){
    				var i = speed;
    				if(endalpha - a < speed && d == 1){
    					i = endalpha - a;
    				}else if(alpha < speed && d == -1){
    					i = a;
    				}
    				alpha = a + (i * d);
    				tt.style.opacity = alpha * .01;
    				tt.style.filter = 'alpha(opacity=' + alpha + ')';
    			}else{
    				clearInterval(tt.timer);
    				if(d == -1){tt.style.display = 'none'}
    			}
    		},
    		hide:function(){
    			clearInterval(tt.timer);
    			tt.timer = setInterval(function(){tooltip.fade(-1)},timer);
    		}
    	}
    }
});

_.extend(OvalForecastView.prototype, BaseMapControlView);

module.exports = OvalForecastView;