'use strict';

var tour = require('app/tour');
var HomeController = require('app/home-controller');
var VerifyTweetsController = require('app/verify-tweets-controller');
var SettingsController = require('app/settings-controller');
var LandingPageController = require('app/landing-page-controller');
var LoginController = require('app/login-controller');
var LeaderboardController = require('app/leaderboard-controller');
var MobileHeaderView = require('app/mobile-header-view');
var Base = require('app/base');
var MobileBase = require('app/mobile-base');
var App = require('app/app');
var Home = require('app/home');
var MobileWebObservationDecisionView = require('app/mobile-web-observation-decision-view');
var StormTrackerView = require('app/storm-tracker-view');

var Router = Marionette.AppRouter.extend({
	routes: {
		'': 'map',
		'verify-tweets/': 'verifyTweets',
		'space-data/': 'spaceData',
		'leaderboard/': 'leaderboard',
		'settings/': 'settings',
		'landing/': 'landingPage'
	},

	initialize: function() {
		App.commands.setHandler('routeAuroraMap', _.bind(function(data) {
			this.map(data);
			this.navigate('');
		},this));

		App.commands.setHandler('routeVerifyTweets', _.bind(function(data) {
			this.verifyTweets(data);
			this.navigate('verify-tweets/');
		},this));

		App.commands.setHandler('routeSpaceData', _.bind(function(data) {
			this.spaceData(data);
			this.navigate('space-data/');
		},this));

		App.commands.setHandler('routeLeaderboard', _.bind(function(data) {
			this.leaderboard(data);
			this.navigate('leaderboard/');
		},this));

		App.commands.setHandler('routeSettings', _.bind(function(data) {
			this.navigate('settings/');
			this.settings(data);
		},this));

		App.commands.setHandler('routeLanding', _.bind(function(data) {
			this.landingPage(data);
			this.navigate('landing/');
		},this));

		App.commands.setHandler('routeReportASighting', _.bind(function(data) {
			this.report();
			this.navigate('sighting/');
		},this));
	},

	map: function() {
		var initialized = !!this.homeController;

		if(initialized) {
			this.homeController.showPage();
		} else {
			this.homeController = new HomeController();
			this.homeController.showPage();

			tour.init();
			tour.start();
		}
	},

	verifyTweets: function() {
		window.analytics.trackView('Verify Tweets');

		this.verifyController = this.verifyController || new VerifyTweetsController();
		this.verifyController.showPage();
	},

	spaceData: function(){
		window.analytics.trackView('Space Data');

		App.main.show(new StormTrackerView({
			model: new Backbone.Model()
		}));
	},

	leaderboard: function(){
		window.analytics.trackView('Leaderboard');

		this.leaderboardController = this.leaderboardController || new LeaderboardController();
		this.leaderboardController.showPage();
	},

	settings: function(){
		if(Aur.authenticated) {
			var controller = new SettingsController();
			controller.showPage();
		} else {
			var controller = new LoginController();
			controller.showPage();
		}
	},

	landingPage: function(){
		window.analytics.trackView('Home');
		var initialized = !!this.landingPageController;

		if(!initialized) {
			$('header').html(new MobileHeaderView({model: new Backbone.Model()}).render().el);
			this.landingPageController = new LandingPageController();
		}		

		this.landingPageController.showPage();
	},

	report: function() {
		var view = new MobileWebObservationDecisionView({model: new Backbone.Model()});
		App.main.show(view);
	}

});

new Base();
new MobileBase();
new Router();

Backbone.history.start({root: '/', silent: true});
Backbone.history.navigate('landing/', {trigger: true});