'use strict';

var App = require('app/app');
var C = require('app/consts');

var BaseMapControlView = require('app/base-map-control-view');

var ScientistFilterView = Marionette.ItemView.extend({
    tagName: 'li',
    className: 'filter-toggle',

    template: Handlebars.compile(
        '<span >Scientist</span>'+
        '<span class="first-legend-icon scientist-icon"></span>'
    ),

    markerClickedEvent: 'scientistMarkerClicked',

    updateMessage: 'ScientistFilterUpdate',
    isActiveMessage: 'ScientistFilterIsActive',
    toggleMessage: 'ScientistFilterToggle',

    unselectedIconUrl: C.get('host') + '/static/img/PinScientist.png',
    selectedIconUrl: C.get('host') + '/static/img/PinScientist-highlight.png',

    initialize: function() {
        this.onInitialize();
    },

    initHandlers: function() {
        App.vent.on('mapDateChanged', _.bind(function(dateString) {
            this.currentMoment = moment(dateString);
            if(this.isActive()) {
                this.removeLayer();
                this.getLayerData();
            }
        }, this));
    },

    onRender: function() {
        this.$el.click(this.toggleLayer.bind(this));
    },

    removeLayer: function() {
        this.el.classList.remove('active');
        this.mapHelper.removeScientistLayer();
        App.vent.trigger(this.updateMessage, false);
    },

    getLayerData: function() {
        this.collection.fetch().done(_.bind(function() {
            this.showLayer();
        }, this));
    },

    showLayer: function() {
        var scientists = this.collection.toJSON(),
            markerOpts = {
            icon: {
                url: this.unselectedIconUrl,
                scaledSize: new google.maps.Size(35, 35)
            }
        };
        this.el.classList.add('active');
        App.vent.trigger(this.updateMessage, true);
        this.mapHelper.addMarkers(scientists, markerOpts);
    }
});

_.defaults(ScientistFilterView.prototype, BaseMapControlView);

module.exports = ScientistFilterView;