'use strict';

var App = require('app/app');
var SettingsView = require('app/settings-view');
var UserProfile = require('app/user-profile');
var BadgeCollection = require('app/badge-collection');
var RecentActivityCollection = require('app/recent-activity-collection');
var ProfileLayout = require('app/profile-layout');
var ProfilePictureItemView = require('app/profile-picture-item-view');
var UserProfileDetailsView = require('app/user-profile-details-view');
var UserProfileEditView = require('app/user-profile-edit-view');
var RecentActivityCompositeView = require('app/recent-activity-composite-view');
var ProfileBadgesView = require('app/profile-badges-view');

module.exports = Marionette.Controller.extend({

	layout: {},

	initialize: function(config) {
		this.config = config;

		App.commands.setHandler('routeSettings', function() {
            this.showPage();
            Backbone.history.navigate('settings/', {});
        }.bind(this));

        App.commands.setHandler('editProfileSettings', function() {
            this.beginSettingsEdit();
        }.bind(this));

        App.commands.setHandler('doneEditingProfileSettings', function() {
            this.endSettingsEdit();
        }.bind(this));
	},

	showPage: function() {
		var fetches = [];
		this.badgeCollection = new BadgeCollection();
		this.recentActivityCollection = new RecentActivityCollection();
		this.userProfile = new UserProfile();

		fetches.push(this.userProfile.fetch());
		fetches.push(this.badgeCollection.fetch());
		fetches.push(this.recentActivityCollection.fetch());

		$.when.apply(this, fetches).done(_.bind(function() {
			this._showPage();
		}, this));
	},

	_showPage: function() {
		this.layout = new ProfileLayout();

		// Either render in "main" region (mobile) or on some passed in element (desktop)
		if(this.config && this.config.$container) {
			this.config.$container.html(this.layout.render().el);
		} else {
			App.main.show(this.layout);
		}

		this.layout.leftCol.show(new ProfilePictureItemView({model: this.userProfile}));
		this.layout.firstSection.show(new UserProfileDetailsView({ model: this.userProfile}));
		this.layout.secondSection.show(new ProfileBadgesView({model: this.badgeCollection}));
		this.layout.thirdSection.show(new RecentActivityCompositeView({
			model: new Backbone.Model(),
			collection: this.recentActivityCollection
		}));
	},

	beginSettingsEdit: function(){
		this.layout.firstSection.show(new UserProfileEditView({ model: this.userProfile }));
		this.layout.secondSection.reset();
		this.layout.thirdSection.reset();
	},

	endSettingsEdit: function(){
		this.showPage();
	}
});