'use strict';

var App = require('app/app');
var MapHelper = require('app/map-helper');
var Utils = require('app/utils');
var TwitterObservationCollection = require('app/twitter-observation-collection');
var WebObservationCollection = require('app/web-observation-collection');

module.exports = Marionette.ItemView.extend({
    tagName: 'li',
    className: 'filter-toggle',
    template: Handlebars.compile('<span id="combined-heatmap">Citizen Estimates</span>'),

    updateMessage: 'twitterVerifiedFilterViewUpdate',
    isActiveMessage: 'signtingHeatmapIsActive',
    toggleMessage: 'signtingHeatmapToggle',

    initialize: function() {
        this.currentMoment = moment();

        this.posWebObsCollection = new WebObservationCollection();
        this.negWebObsCollection = new WebObservationCollection();
        this.negTwitterObsCollection = new TwitterObservationCollection();
        this.posTwitterObsCollection = new TwitterObservationCollection();

        this.mapHelperPos = new MapHelper();
        this.mapHelperNeg = new MapHelper();

        this.getHeapmapData();

        App.commands.setHandler(this.toggleMessage, function() {
            this.toggleHeatmap();
        }.bind(this));
        
        App.reqres.setHandler(this.isActiveMessage, function() {
            return this.$el.hasClass('active');
        }.bind(this));
    },

    onRender: function() {
        this.$el.click(this.toggleHeatmap.bind(this));
    },

    getHeapmapData: function() {
        var $fetches = [],
            page = 1,
            posOptions = {
                gradient: [
                    'rgba(255, 255, 255, 0)',
                    'rgba(150, 255, 150, 1)',
                    'rgba(130, 255, 130, 1)',
                    'rgba(65, 255, 30, 1)',
                    'rgba(58, 255, 10, 1)'
                ]
            },
            negOptions = { 
                gradient: [
                    'rgba(255, 255, 255, 0)',
                    'rgba(255, 150, 150, 1)',
                    'rgba(255, 130, 130, 1)',
                    'rgba(255, 65, 30, 1)',
                    'rgba(255, 58, 10, 1)'
                ]
            };

        this.el.classList.add('active');

        this.posWebObsCollection.queryParams.end_date = this._getTime();
        this.posWebObsCollection.queryParams.local_offset = this._getLocalOffset();
        this.posWebObsCollection.queryParams.local_offset = this._getLocalOffset();
        this.posWebObsCollection.queryParams.pos_sighting = true;
        $fetches.push(this.posWebObsCollection.getPage(page, {reset: true}));

        this.negWebObsCollection.queryParams.end_date = this._getTime();
        this.negWebObsCollection.queryParams.local_offset = this._getLocalOffset();
        this.negWebObsCollection.queryParams.neg_sighting = true;
        $fetches.push(this.negWebObsCollection.getPage(page, {reset: true}));

        this.negTwitterObsCollection.queryParams.end_date = this._getTime();
        this.negTwitterObsCollection.queryParams.local_offset = this._getLocalOffset();
        this.negTwitterObsCollection.queryParams.verified_neg = true;
        $fetches.push(this.negTwitterObsCollection.getPage(page, {reset: true}));

        this.posTwitterObsCollection.queryParams.end_date = this._getTime();
        this.posTwitterObsCollection.queryParams.local_offset = this._getLocalOffset();
        this.posTwitterObsCollection.queryParams.verified_pos = true;
        $fetches.push(this.posTwitterObsCollection.getPage(page, {reset: true}));

        $.when.apply(this, $fetches).done(function(posWeb, negWeb, posTwt, negTwt) {
            var pos = posWeb[0].results;
            pos.concat(posTwt[0].results);
            var neg = negWeb[0].results;
            neg.concat(negTwt[0].results);

          this.mapHelperPos.createHeatmap(pos, posOptions);
          this.mapHelperNeg.createHeatmap(neg, negOptions);

        }.bind(this));
    },

    removeHeatmaps: function() {
        this.mapHelperPos.removeHeatmapLayer();
        this.mapHelperNeg.removeHeatmapLayer();
        this.el.classList.remove('active');
    },

    toggleHeatmap: function() {
        if(this.$el.hasClass('active')){
            this.removeHeatmaps();
            App.execute(this.updateMessage, false);
        }
        else{
            this.getHeapmapData();
            App.execute(this.updateMessage, true);
        }
    },

    _getTime: function() {
        var nowRounded = Utils.roundXMinutes(this.currentMoment.clone(), 1);
        return nowRounded.format(Utils.defaultDateFmtWithTZ());
    },

    _getLocalOffset: function() {
        return this.currentMoment.toDate().getTimezoneOffset() * 60;
    }

});