'use strict';

var C = require('app/consts');

var SpaceWeatherChart = function(config) {
	this.$selector = config.$selector;
}

SpaceWeatherChart.prototype._fetchData = function() {
	return $.getJSON(C.get('host') + '/space-data');
}

SpaceWeatherChart.prototype.graph = function() {
	this._fetchData()
	.done(_.bind(function(data) {
		this._parseResponse(data);
		this._graph();
	}, this));
};

SpaceWeatherChart.prototype._graph = function() {
	var startDateFormatted =  moment(this.startDate).format('MMM D h:mm A')
	var endDateFormatted = moment(this.endDate).format('MMM D h:mm A')

	this.$selector.highcharts({
	    chart: {
	        zoomType: 'x',
	        spacingRight: 20
	    },
	    title: {
	        text: 'Solar Wind Power vs Time: ' + startDateFormatted + ' - ' + endDateFormatted
		 	},
	    subtitle: {
	        text: document.ontouchstart === undefined ?
	            'Click and drag in the plot area to zoom in' :
	            'Pinch the chart to zoom in'
	    },
	    xAxis: {
	        type: 'datetime',
	        minRange: 1000 * 60,
	        title: {
	            text: 'Time (UTC)'
	        }
	    },
	    yAxis: {
	    	min:0,
	        title: {
	            text: 'Solar wind power (GW)'
	        },

	        labels: {
	        	formatter: function() {
	        		var description = '';
	        		var gw = this.value / 1000000000;
	        		if(gw === 50) description = '(substorm)';
	        		if(gw === 200) description = '(small storm)';
	        		if(gw === 1000) description = '(medium storm)';
	        		if(gw === 4000) description = '(large storm)';
	        		if(gw === 10000) description = '(very large storm)';
	        		return gw + ' GW ' + description;
	        	}
	        }
	    },
	    tooltip: {
	        shared: true,
	        valueDecimals: 0,
	        valueSuffix: 'W',
	    },
	    legend: {
	        enabled: false
	    },
	    plotOptions: {
	        area: {
	            lineWidth: 1,
	            marker: {
	                enabled: false
	            },
	            shadow: false,
	            states: {
	                hover: {
	                    lineWidth: 1
	                }
	            },
	            threshold: null
	        }
	    },

	    series: [{
	        type: 'area',
	        name: 'Epsilon',
	        pointInterval: 60 * 1000,
	        pointStart: this.startDate.getTime(),
	        data: this.stormData

	    }]
	});	
}

SpaceWeatherChart.prototype._parseResponse = function(data) {
	this.stormData = data.storm_data;
	this.startDate = new Date(data.start_date);
	this.endDate = new Date(data.end_date);
}

module.exports = SpaceWeatherChart;