'use strict';

var C = require('app/consts');
var App = require('app/app');
var Utils = require('app/utils');
var TweetVoteItemView = require('app/tweet-vote-item-view');
var MobileNav = require('app/mobile-nav');

var tweetVoteCompositeView = Marionette.CompositeView.extend({
	template: Handlebars.compile(
		'<div class="row pageTitle">'+
			'<h2 class="large-12 columns">{{#ifIsMobile}}<span id="mobileBackButton"><img src="img/NavBackIcon.png"></span>{{/ifIsMobile}}VERIFY TWEETS</h2>'+
		'</div>'+
		'<div id="tweets-container" class="row"></div>'
	),

	childView: TweetVoteItemView,
	childViewContainer: '#tweets-container',

	ui:{
		backNav: '#mobileBackButton'
	},

	initialize: function() {
		var initialSize = this.collection.length;
		this.minSize = C.get('VERIFY_TWEETS_COLLECTION_SIZE');
		this.currentPage = 1;
		this.ableToFetchMore = (initialSize >= this.minSize);
		this.initHandlers();
	},

	initHandlers: function() {
		this.on('childview:voted', function(view) {
			this.collection.remove(view.model);

			if(this.ableToFetchMore && (this.minSize - this.collection.length >= 5)) {
				this.currentPage++;
				var oldestMoment = moment(this.collection.last().get('created_at'));
				var params = {
					end_date: oldestMoment.format(Utils.defaultDateFmtWithTZ()),
					page_size: this.minSize,
					page: this.currentPage,
					order_by: '-created_at'
				};
				$.ajax({
					data: $.param(params), silent: true,
					url: C.get('host') + '/tweets'
				}).done(function(data) {
					this.collection.add(data.results);
					this.collection.sort();
					if(!data || !data.results.length) {
						this.ableToFetchMore = false;
					}
				}.bind(this));
			}
		}.bind(this));
	},

	onRender: function(){
		if (window.cordova){
			this.ui.backNav.on('click', MobileNav.navBack);
		}
	}
});

_.extend(tweetVoteCompositeView.prototype, MobileNav);
module.exports = tweetVoteCompositeView;
