'use strict';

var App = require('app/app');
var TwitterVote = require('app/twitter-vote');
var MobileUtils = require('app/mobile-utils');

module.exports = Marionette.ItemView.extend({
	className: 'large-12 columns twitter-status',

	template: aur.tpl['tweet-vote-item-view-tpl'],

	ui: {
		voteYesInput: '.affirmativeButton',
		voteNoInput: '.negativeButton',
		votingHelp: '.twitter-vote-help'
	},

	events: {
		'click @ui.voteYesInput ': 'vote',
		'click @ui.voteNoInput ': 'vote',
		'click a': 'openURL',
		'click @ui.votingHelp': 'showVotingHelp'
	},

	initialize: function() {
		this.setBiggerTwitterUserImg();
		this.voteModel = new TwitterVote({observation: this.model.get('id')});
	},

	setBiggerTwitterUserImg: function() {
		var imageUrl = this.model.get('user_profile_image_url');
		imageUrl = imageUrl.replace('_normal\.', '_bigger.');
		this.model.set('user_profile_image_url', imageUrl);
	},

	vote: function(e) {
		var voteType = $(e.currentTarget).data('vote');
		
		this.voteModel.set('vote_type', voteType);
		this.voteModel.save(null, {
			global: false
		});
		
		this.$el.css('max-height', 0);
		this.$el.css('padding-top', 0);
		this.$el.css('padding-bottom', 0);

		// Remove the view once the transition is complete
		setTimeout(function() {
			this.destroy();
			this.trigger('voted');
		}.bind(this), 350);
	},

	openURL: function(e) {
		if(window.cordova) {
			e.preventDefault();
			MobileUtils.openURL(e.currentTarget.href);
		}
	},

	showVotingHelp: function() {
		if(window.cordova) {
			navigator.notification.alert('Your help finding real observations '+
				'on Twitter when activity is high will help Aurorasaurus generate '+
				'accurate nowcast predictions. Click YES if the tweet is a real '+
				'sighting from a person who has just seen aurora AND if the location '+
				'appears correct. Click NO if it is a retweet, clearly has the wrong '+
				'location, OR is a tweet about ANYTHING else related to the words '+
				'Northern Lights. If you aren’t sure, click NO, but if you are '+
				'reasonably sure, please click YES.', 
                _.bind(function() {
                }, this),
                'How To Vote'
            );
		}
	}
});