'use strict';

var App = require('app/app');
var MapHelper = require('app/map-helper');
var BaseView = require('app/base-view');

var TweetVoteView = Marionette.ItemView.extend({
    template: aur.tpl['tweet-vote-view-tpl'],

    ui:{
    	upVoteInput: '#upVote',
    	downVoteInput: '#downVote',
    	confirmVote: '#mapDrawerForm',
        questionButton: '#questionButton',
        messages: '#messages'
    },

    events:{
    	'click @ui.downVoteInput': 'updateAndSaveModel',
    	'click @ui.upVoteInput': 'updateAndSaveModel',
        'click @ui.questionButton': 'showHelp'
    },

    successMsg: 'Thanks for voting!',

    initialize: function() {
    	this.model.set('observation', this.model.get('tweet').id);
    },

    updateAndSaveModel: function(e) {
        var voteType = $(e.currentTarget).data('vote');

    	this.model.set('vote_type', voteType);
        this.instantSubmit();

        this.ui.upVoteInput.css('display', 'none');
        this.ui.downVoteInput.css('display', 'none');
    },

    showVotingHelp: function() {
        navigator.notification.alert('Some text explaining how to vote on Tweets', 
            _.bind(function() {
            }, this),
            'How To Vote'
        );
    },

    showHelp: function(e){
        if(window.cordova) {
            this.showMobileHelp();
        } else {
            if ($('#helpPopup').css('display') == 'none'){
                $('#helpPopup').show();
            }
            else{
                $('#helpPopup').hide();
            }
        }
    },

    showMobileHelp: function() {
        navigator.notification.alert('Your help finding real observations '+
            'on Twitter when activity is high will help Aurorasaurus generate '+
            'accurate nowcast predictions. Click YES if the tweet is a real '+
            'sighting from a person who has just seen aurora AND if the location '+
            'appears correct. Click NO if it is a retweet, clearly has the wrong '+
            'location, OR is a tweet about ANYTHING else related to the words '+
            'Northern Lights. If you aren’t sure, click NO, but if you are '+
            'reasonably sure, please click YES.', 
            _.bind(function() {
            }, this),
            'How To Vote'
        );
    },

    onHandleError: function(model, response) {
        if(response.status === 400) return 'You have already voted for this observation';
    }

});

_.extend(TweetVoteView.prototype, BaseView);

module.exports = TweetVoteView;