'use strict';

var C = require('app/consts');
var TwitterFilterBaseView = require('app/twitter-filter-base-view')

var TwitterUnverifiedFilterView = Marionette.ItemView.extend({
	tagName: 'li',
    className: 'filter-toggle',
    template: Handlebars.compile(
        '<span id="twitter-reports-layer-input">Unverified Tweet</span>'+
        '<span class="first-legend-icon unverified-icon"></span>'+
        '<span class="second-legend-icon unverified-group-icon"></span>'+
        '<span class="legend-icon-group-text">10+</span>'
    ),

    markerClickedEvent: 'unverifiedTwitterMarkerClicked',
    
    updateMessage: 'twitterUnverifiedFilterViewUpdate',
    isActiveMessage: 'twitterUnverifiedFilterViewIsActive',
    toggleMessage: 'twitterUnverifiedFilterViewToggle',

    selectedIconUrl: C.get('host') + '/static/img/PinUnverifiedTweetHighlight.png',
    unselectedIconUrl: C.get('host') + '/static/img/PinUnverifiedTweet.png',

    initialize: function() {
        this.onInitialize();
    },

    onRender: function() {
        this.$el.click(this.toggleLayer.bind(this));
    },

    showLayer: function() {
        var tweetObservations = this.collection.toJSON();
        if(tweetObservations[0] && tweetObservations[0].results) tweetObservations = tweetObservations[0].results;
        var options = {
            gridSize: C.get('CLUSTER_GRID_SIZE'),
            maxZoom: 15,
            minimumClusterSize: C.get('CLUSTER_SIZE'),
            styles: [{
                url: C.get('host') + '/static/img/PinUnverifiedTweetGroup.png',
                width: 35,
                height: 35
            }],
        },
        markerOpts = {
            icon: {
                url: this.unselectedIconUrl,
                scaledSize: new google.maps.Size(35, 35)
            }
        };
    	this.mapHelper.renderCluster(tweetObservations, options, markerOpts);
    },

    getLayerData: function(page) {
        if(!page) page = 1;

        this.collection.queryParams.end_date = this._getTime();
        this.collection.queryParams.local_offset = this._getLocalOffset();
        this.collection.getPage(page, {reset: true})
        .done(function() {
            if(this.collection.hasNextPage()) this.getLayerData(page + 1);
        }.bind(this));
        this.el.classList.add('active');
    }
});

_.extend(TwitterUnverifiedFilterView.prototype, TwitterFilterBaseView);

module.exports = TwitterUnverifiedFilterView;