'use strict'

var App = require('app/app');

module.exports = Marionette.ItemView.extend({

	template: aur.tpl['user-profile-edit-view-tpl'],

	ui: {
		emailInput: "#email",
		locationInput: "#location",
		emailAlertCheckbox: "#notifications", 
		twitterMessagesCheckbox: "#twitterMessages", 
		doneEditing: '#doneEditingButton',
		cancel: '#cancelEditBtn',
        mobileDoneEditing: '#mobileSaveProfile',
        mobileCancel: '#mobileCancelSaveProfile'
	},

	events: {
		'change @ui.emailInput': 'updateModel',
		'change @ui.emailAlertCheckbox': 'updateModel',
		'change @ui.twitterMessagesCheckbox': 'updateModel',
		'click @ui.doneEditing': 'doneEditingWasPressed',
		'click @ui.cancel': 'cancelEdit',
        'click @ui.mobileDoneEditing': 'doneEditingWasPressed',
        'click @ui.mobileCancel': 'cancelEdit'
	},
	
	onRender: function() {
    	this.initLocationSearchInput();
    },

    initLocationSearchInput: function() {
        this.ui.locationInput.keypress(function(e) {
            if (e.which == 13) {
                return false;
            }
        });

    	var searchBox = new google.maps.places.SearchBox(this.ui.locationInput[0]);
        google.maps.event.addListener(searchBox, 'places_changed', function() {
            var place = searchBox.getPlaces()[0];
            var loc = {};
            loc['name'] = place.name;
            loc['address'] = place.formatted_address;
            loc['latitude'] = place.geometry.location.lat();
            loc['longitude'] = place.geometry.location.lng();
            loc['google_reference'] = place.reference;
            loc['google_id'] = place.id;
            loc['user_profile'] = this.model.get('id');
            this.model.set('location', loc);
        }.bind(this));
    },


	updateModel: function() {
        this.model.get('user').email = this.ui.emailInput.val();
        this.model.set('receives_email_alerts', this.ui.emailAlertCheckbox.is(':checked'));
        this.model.set('receives_twitter_messages', this.ui.twitterMessagesCheckbox.is(':checked'));
    },

	doneEditingWasPressed: function(){
		this.model.save().done(function(){
			App.execute('doneEditingProfileSettings');
		});
	},

	cancelEdit: function() {
		App.execute('doneEditingProfileSettings');
	}

});