'use strict';

var Utils = require('app/utils');
var C = require('app/consts');

module.exports = Backbone.Model.extend({

	urlRoot: C.get('host') + '/web-obs',

	name: 'web-observation',
	
	defaults: {
		activities: null,
		height: null,
    	colors: [],
    	types: [],
    	sky: null,

    	imageUrl: null,

    	address_country: null,
    	address_state: null,

	    see_aurora: null,
	    location: null,

	    time_start: function() {
	    	return Utils.round15Minutes(moment()).format(Utils.defaultDateFmtWithTZ());
	    }.call(this),
	    time_end: function() {
	    	var end = this.time_start;
	    	if(end) end = Utils.round15Minutes(moment(end));
	    	else end = Utils.round15Minutes(moment());
	    	end.add('minutes', 15);
	    	return end.format(Utils.defaultDateFmtWithTZ());
	    }.call(this),

	    on_going: false,

	    height_other: null,
	    activities_other: null,
	    colors_other: null,
	    types_other: null,
	    sky_other: null,
	    comment: null
	},

	validation: {
		activities: {
			required: false
		},
    	colors: {
			required: false
		},
    	types: {
			required: false
		},
    	height: {
			required: false
		},
    	sky: {
			required: false
		},

    	address_country: {
			required: false
		},
    	address_state: {
			required: false
		},

	    see_aurora: {
			required: true
		},
	    sky_other: {
			required: false,
			maxLength: 1024
		},

	    time_start: function(value, attr, computedState) {
			if( value && (moment(this.get('time_end')).isBefore(moment(value))) ) {
				return 'Start time must be before end time';
			}
		},
	    time_end: function(value, attr, computedState) {
	    	if( value && (moment(this.get('time_start')).isAfter(moment(value))) ) {
				return 'End time must be after start time';
			}	
		},

		location: {
			required: true
		},

	    on_going: {
			required: true
		},

	    height_other: {
			required: false,
			maxLength: 1024
		},
	    activities_other: {
			required: false,
			maxLength: 1024
		},
	    colors_other: {
			required: false,
			maxLength: 1024
		},
	    types_other: {
			required: false,
			maxLength: 1024
		},
	    comment: {
			required: false,
			maxLength: 4096
		}
	}
});